@@ -3,6 +3,7 @@ import inspect
 import itertools
 import signal
 import sys
+import threading
 import warnings
 
 import decorator
@@ -200,7 +201,16 @@ def init_twisted_greenlet():
         return
 
     if not _instances.reactor.running:
-        if signal.getsignal(signal.SIGINT) == signal.default_int_handler:
+        if not isinstance(threading.current_thread(), threading._MainThread):
+            warnings.warn(
+                (
+                    'Will not attempt to block Twisted signal configuration'
+                    ' since we are not running in the main thread.  See'
+                    ' https://github.com/pytest-dev/pytest-twisted/issues/153.'
+                ),
+                RuntimeWarning,
+            )
+        elif signal.getsignal(signal.SIGINT) == signal.default_int_handler:
             signal.signal(
                 signal.SIGINT,
                 functools.partial(signal.default_int_handler),
@@ -365,27 +375,51 @@ def pytest_pyfunc_call(pyfuncitem):
     # TODO: only handle 'our' tests?  what is the point of handling others?
     #       well, because our interface allowed people to return deferreds
     #       from arbitrary tests so we kinda have to keep this up for now
-    _run_inline_callbacks(_async_pytest_pyfunc_call, pyfuncitem)
-    return not None
+    maybe_hypothesis = getattr(pyfuncitem.obj, "hypothesis", None)
+    if maybe_hypothesis is None:
+        _run_inline_callbacks(
+            _async_pytest_pyfunc_call,
+            pyfuncitem,
+            pyfuncitem.obj,
+            {}
+        )
+        result = not None
+    else:
+        hypothesis = maybe_hypothesis
+        f = hypothesis.inner_test
+
+        def inner_test(**kwargs):
+            return _run_inline_callbacks(
+                _async_pytest_pyfunc_call,
+                pyfuncitem,
+                f,
+                kwargs,
+            )
+
+        pyfuncitem.obj.hypothesis.inner_test = inner_test
+        result = None
+
+    return result
 
 
 @defer.inlineCallbacks
-def _async_pytest_pyfunc_call(pyfuncitem):
+def _async_pytest_pyfunc_call(pyfuncitem, f, kwargs):
     """Run test function."""
-    kwargs = {
+    fixture_kwargs = {
         name: value
         for name, value in pyfuncitem.funcargs.items()
         if name in pyfuncitem._fixtureinfo.argnames
     }
+    kwargs.update(fixture_kwargs)
 
-    maybe_mark = _get_mark(pyfuncitem.obj)
+    maybe_mark = _get_mark(f)
     if maybe_mark == 'async_test':
-        result = yield defer.ensureDeferred(pyfuncitem.obj(**kwargs))
+        result = yield defer.ensureDeferred(f(**kwargs))
     elif maybe_mark == 'inline_callbacks_test':
-        result = yield pyfuncitem.obj(**kwargs)
+        result = yield f(**kwargs)
     else:
         # TODO: maybe deprecate this
-        result = yield pyfuncitem.obj(**kwargs)
+        result = yield f(**kwargs)
 
     defer.returnValue(result)
 
